#include "velomeca.h"
#include <gmt.h>

/***********************************************************************************************************/
double  ps_mechanism(x0, y0, meca, scale, rgb, outline)

/* Genevieve Patau */

double x0, y0;
st_me meca;
double scale;
int rgb[3];
BOOLEAN outline;
{

    double null_axis_dip();
    double null_axis_strike();
    double proj_radius();
    double zero_360();
    double computed_mw();

    double x[1000], y[1000];
    double pos_NP1_NP2 = sind(meca.NP1.str - meca.NP2.str);
    double fault = meca.NP1.rake == 0. ? meca.NP2.rake / fabs(meca.NP2.rake) : meca.NP1.rake / fabs(meca.NP1.rake);
    double radius_size;
    double str, radius, increment;

    int white[3], lineout = 1, i, npoints;

    struct AXIS N_axis;
    struct MOMENT moment;

    white[0] = 255;
    white[1] = 255;
    white[2] = 255;
   
    moment.mant = meca.moment.mant;
    moment.exponent = meca.moment.exponent;

/* compute null axis strike and dip */
    N_axis.dip = null_axis_dip(meca.NP1.str, meca.NP1.dip, meca.NP2.str, meca.NP2.dip);
    if(fabs(90. - N_axis.dip) < EPSIL) 
        N_axis.str = meca.NP1.str;
    else
        N_axis.str = null_axis_strike(meca.NP1.str, meca.NP1.dip, meca.NP2.str, meca.NP2.dip);
 
/* compute radius size of the bubble */
    radius_size = computed_mw(moment, meca.magms) / 5. * scale;
/* outline the bubble */
    ps_plot(x0 + radius_size, y0, 3); 
/*  argument is DIAMETER!!*/
    ps_circle(x0, y0, radius_size*2., white, lineout); 
 
    if(fabs(pos_NP1_NP2) < EPSIL) {
/* pure normal or inverse fault (null axis strike is determined
   with + or - 180 degrees. */
        /* first nodal plane part */
        i = -1;
        increment = 1;
        str = meca.NP1.str;
        while(str <= meca.NP1.str + 180.) {
            i++;
            radius = proj_radius(meca.NP1.str, meca.NP1.dip, str) * radius_size;
            x[i] = x0 + radius * sind(str);
            y[i] = y0 + radius * cosd(str);
            str += increment;
        }
        if(fabs(fault + 1.) < EPSIL) {
            /* normal fault, close first compressing part */
            str = meca.NP1.str + 180.; 
            while(str >= meca.NP1.str) {
                i++;
                x[i] = x0 + sind(str) * radius_size;
                y[i] = y0 + cosd(str) * radius_size;
                str -= increment;
            }
            npoints = i + 1;
            ps_polygon(x, y, npoints, rgb, outline);
            i = -1;
        }
        /* second nodal plane part */
        str = meca.NP2.str;
        while(str <= meca.NP2.str + 180.) {
            i++;
            radius = proj_radius(meca.NP2.str, meca.NP2.dip, str) * radius_size;
            x[i] = x0 + radius * sind(str);
            y[i] = y0 + radius * cosd(str);
            str += increment;
        }
        if(fabs(fault - 1.) < EPSIL) {
            /* inverse fault, close compressing part */
            npoints = i+1;
            ps_polygon(x, y, npoints, rgb, outline);
        }
        else {
            /* normal fault, close second compressing part */
            str = meca.NP2.str + 180.;
            while(str >= meca.NP2.str) {
                i++;
                x[i] = x0 + sind(str) * radius_size;
                y[i] = y0 + cosd(str) * radius_size;
                str -= increment;
            } 
            npoints = i + 1;
            ps_polygon(x, y, npoints, rgb, outline);
        }
    }
/* pure strike-slip */
    else if((90. - meca.NP1.dip) < EPSIL && (90. - meca.NP2.dip) < EPSIL) {
        increment = fabs(meca.NP1.rake) < EPSIL ? 1. : -1.;
        /* first compressing part */
        i = 0; 
        str = meca.NP1.str;
        while(increment == 1 ? str <= meca.NP1.str + 90. : str >= meca.NP1.str - 90.) {
            x[i] = x0 + sin_d(str) * radius_size;
            y[i] = y0 + cos_d(str) * radius_size;
            str += increment;
            i++;
        }
        x[i] = x0;
        y[i] = y0;
        npoints = i + 1;
        ps_polygon(x, y, npoints, rgb, outline);
        /* second compressing part */
        i = 0;
        str = meca.NP1.str + 180.;
        while(increment == 1 ? str <= meca.NP1.str + 270. : str >= meca.NP1.str + 90.) {
            x[i] = x0 + sin_d(str) * radius_size;
            y[i] = y0 + cos_d(str) * radius_size;
            str += increment;
            i++;
        }
        x[i] = x0;
        y[i] = y0;
        npoints = i + 1;
        ps_polygon(x, y, npoints, rgb, outline);
    }
    else {
/* other cases */
        /* first nodal plane till null axis */
        i = -1;
        increment = 1;
        if(meca.NP1.str > N_axis.str)
            meca.NP1.str -= 360.;
        str = meca.NP1.str;
        while(fabs(90. - meca.NP1.dip) < EPSIL ? str <= meca.NP1.str : str <= N_axis.str) {
            i++;
            radius = proj_radius(meca.NP1.str, meca.NP1.dip, str) * radius_size;
            x[i] = x0 + radius * sind(str);
            y[i] = y0 + radius * cosd(str);
            str += increment;
        }    
        
        /* second nodal plane from null axis */ 
        meca.NP2.str += (1. + fault) * 90.;
        if(meca.NP2.str >= 360.)
            meca.NP2.str -= 360.;
        increment = fault;
        if(fault * (meca.NP2.str - N_axis.str) < -EPSIL)
            meca.NP2.str += fault * 360.;
        str = fabs(90. - meca.NP2.dip) < EPSIL ? meca.NP2.str : N_axis.str;
        while(increment == 1. ? str <= meca.NP2.str : str >= meca.NP2.str) {
            i++;
            radius = proj_radius(meca.NP2.str - (1. + fault) * 90., meca.NP2.dip, str) * radius_size;
            x[i] = x0 + radius * sind(str);
            y[i] = y0 + radius * cosd(str);
            str += increment;
        }

        /* close the first compressing part */
        meca.NP1.str = zero_360(meca.NP1.str);
        meca.NP2.str = zero_360(meca.NP2.str);
        increment = pos_NP1_NP2 >= 0. ? -fault : fault;
        if(increment * (meca.NP1.str - meca.NP2.str) < - EPSIL)
            meca.NP1.str += increment * 360.;
        str = meca.NP2.str;
        while(increment == 1. ? str <= meca.NP1.str : str >= meca.NP1.str) {
            i++;
            x[i] = x0 + sind(str) * radius_size;
            y[i] = y0 + cosd(str) * radius_size;
            str += increment;
        }

        npoints = i + 1;
        ps_polygon(x, y, npoints, rgb, outline);

        /* first nodal plane till null axis */
        i = -1;
        meca.NP1.str = zero_360(meca.NP1.str + 180.);
        if(meca.NP1.str - N_axis.str < - EPSIL)
            meca.NP1.str += 360.;
        increment = -1.;
        str = meca.NP1.str;
        while(fabs(90. - meca.NP1.dip) < EPSIL ? str >= meca.NP1.str : str >= N_axis.str) {
            i++;
            radius = proj_radius(meca.NP1.str - 180., meca.NP1.dip, str) * radius_size;
            x[i] = x0 + radius * sind(str);
            y[i] = y0 + radius * cosd(str);
            str += increment;
        }

        /* second nodal plane from null axis */
        meca.NP2.str = zero_360(meca.NP2.str + 180.);
        increment = -fault;
        if(fault * (N_axis.str - meca.NP2.str) < - EPSIL)
            meca.NP2.str -= fault * 360.; 
        str = fabs(90. - meca.NP2.dip) < EPSIL ? meca.NP2.str : N_axis.str;
        while(increment == 1. ? str <= meca.NP2.str : str >= meca.NP2.str) {
            i++;
            radius = proj_radius(meca.NP2.str - (1. - fault) * 90., meca.NP2.dip, str) * radius_size;
            x[i] = x0 + radius * sind(str);
            y[i] = y0 + radius * cosd(str);
            str += increment;
        }

        /* close the second compressing part */
        meca.NP1.str = zero_360(meca.NP1.str);
        meca.NP2.str = zero_360(meca.NP2.str);
        increment = pos_NP1_NP2 >= 0. ? -fault : fault;
        if(increment * (meca.NP1.str - meca.NP2.str) < - EPSIL)
            meca.NP1.str += increment * 360.;
        str = meca.NP2.str;
        while(increment == 1. ? str <= meca.NP1.str : str >= meca.NP1.str) {
            i++;
            x[i] = x0 + sind(str) * radius_size;
            y[i] = y0 + cosd(str) * radius_size;
            str += increment;
        }

        npoints = i + 1;
        ps_polygon(x, y, npoints, rgb, outline);
    } 
    return(radius_size);
} 

/*********************************************************************/
double ps_meca(x0, y0, meca, scale)

/* Genevieve Patau */

double x0, y0;
st_me meca;
double scale;
{

    int i, npoints;

    double proj_radius();
    double zero_360();
    double computed_mw();

    double x[1000], y[1000];
    double radius_size;
    double str, radius, increment;

    int white[3], lineout = 1;


 
    struct MOMENT moment;

    moment.mant = meca.moment.mant;
    moment.exponent = meca.moment.exponent;

    white[0] = 255;
    white[1] = 255;
    white[2] = 255;


/* compute radius size of the bubble */
    radius_size = computed_mw(moment, meca.magms) / 5. * scale;

/* outline the bubble */
    ps_plot(x0 + radius_size, y0, 3); 
/*    ps_circonf(x0, y0, radius_size); */
/*  argument is DIAMETER!!*/
    ps_circle(x0, y0, radius_size*2., white, lineout);

        i = -1;
        increment = 1;
        str = meca.NP1.str;
        while(str <= meca.NP1.str + 180.) {
            i++;
            radius = proj_radius(meca.NP1.str, meca.NP1.dip, str) * radius_size;   
            x[i] = x0 + radius * sind(str);
            y[i] = y0 + radius * cosd(str);
            str += increment;
        }
        npoints = i + 1;
        ps_line(x, y, npoints, 1, FALSE, FALSE);

        i = -1;
        increment = 1;
        str = meca.NP2.str;
        while(str <= meca.NP2.str + 180.) {
            i++;
            radius = proj_radius(meca.NP2.str, meca.NP2.dip, str) * radius_size;
            x[i] = x0 + radius * sind(str);
            y[i] = y0 + radius * cosd(str);
            str += increment;
        }
        npoints = i + 1;
        ps_line(x, y, npoints, 1, FALSE, FALSE);
        return(radius_size);
}

/*********************************************************************/
double zero_360(str)

  /* put an angle between 0 and 360 degrees */

  /* Genevieve Patau */

double(str);
{
    if(str >= 360.)
        str -= 360.;
    else if(str < 0.)
        str += 360.;
    return(str);
}

/**********************************************************************/
double computed_mw(moment, ms)

  /*
     Compute mw-magnitude from seismic moment or MS magnitude.
 */

  /* Genevieve Patau */

struct MOMENT moment;
double ms;
{
    double mw;

    if(moment.exponent == 0)
        mw = ms;
    else
        mw = (log10(moment.mant) + (double)moment.exponent - 16.5) * 2. / 3.;

    return(mw);
}

/*********************************************************************/
double datan2(y, x)
 
  /*  compute arctg in degrees, between -180 et +180. */

  /* Genevieve Patau */

double y, x;
{
    double arctg;
    double rdeg = 180. / 3.14159265;
  
    if(fabs(x) < EPSIL) {
        if(fabs(y) < EPSIL) {
/*
            printf("indetermined form 0. / 0.");
            exit();
*/
            arctg = 0.;
        }
        else
            arctg = y < 0. ? -90. : 90.;
    }
    else if(x < 0.)
        arctg = y < 0. ? atan(y / x) * rdeg - 180. : atan(y / x) * rdeg + 180.;
    else 
        arctg = atan(y / x) * rdeg;

    return(arctg);
}

/*********************************************************************/
double computed_strike1(NP1)

/*
   Compute the strike of the decond nodal plane when are given
   strike, dip and rake for the first nodal plane with AKI & RICHARD's
   convention.
   Angles are in degrees.
*/

/* Genevieve Patau */

struct nodal_plane NP1;
{
    double str2;
    double cd1 = cosd(NP1.dip), cg1 = cosd(NP1.rake);
    double temp;
    double cp2, sp2;
    double am = NP1.rake == 0 ? 1. : NP1.rake /fabs(NP1.rake); 

    double datan2();
  
    if(cd1 < EPSIL && abs(cg1) < EPSIL) {
/*
        printf("\nThe second plane is horizontal;");
        printf("\nStrike is indetermined.");
        printf("\nstr2 = NP1.str is taken to define");
        printf("\nrake in the second plane.");
*/
        str2 = NP1.str;
    }
    else {
        temp = cg1 * cosd(NP1.str);
        temp += sind(NP1.rake) * sind(NP1.str) * cd1;
        sp2 = -am * temp;
        temp = sind(NP1.str) * cg1;
        temp -= sind(NP1.rake) *  cosd(NP1.str) * cd1;
        cp2 = am * temp;
        str2 = datan2(sp2, cp2);
        str2 = zero_360(str2);
    }
    return(str2);
}

/*********************************************************************/
double computed_dip1(NP1)

/*
   Compute second nodal plane dip when are given strike,
   dip and rake for the first nodal plane with AKI & RICHARD's
   convention.
   Angles are in degrees.
*/

/* Genevieve Patau */

struct nodal_plane NP1;
{
    double am = NP1.rake == 0 ? 1. : NP1.rake / fabs(NP1.rake);
    double dip2;
  
    dip2 = acos(am * sind(NP1.rake) * sind(NP1.dip)) / D2R;

    return(dip2);
}

/*********************************************************************/
double computed_rake1(NP1)

/* 
   Compute rake in the second nodal plane when strike ,dip
   and rake are given for the first nodal plane with AKI & 
   RICHARD's convention.
   Angles are in degrees.
*/

/* Genevieve Patau */

struct nodal_plane NP1;
{
    double computed_strike1();
    double computed_dip1();
    double datan2();
  
    double rake2, sinrake2;
    double str2 = computed_strike1(NP1);
    double dip2 = computed_dip1(NP1);
    double am = NP1.rake == 0. ? 1. : NP1.rake / fabs(NP1.rake);

    if(fabs(dip2 - 90.) < EPSIL)
        sinrake2 = am * cosd(NP1.dip);
    else
        sinrake2 = -am * sind(NP1.dip) * cosd(NP1.str - str2) / cosd(dip2);

    rake2 = datan2(sinrake2, -am * sind(NP1.dip) * sind(NP1.str - str2));

    return(rake2);
}

/*********************************************************************/
double computed_dip2(str1,dip1,str2)

/* 
   Compute second nodal plane dip when are given
   strike and dip for the first plane and strike for 
   the second plane.
   Angles are in degrees.
   Warning : if dip1 == 90 and cos(str1 - str2) == 0
             the second plane dip is indetermined.
*/
 
/* Genevieve Patau */

double str1,str2;
double dip1;
{
    double dip2;
    double cosdp12 = cosd(str1 - str2);

    double datan2();
  
    if(fabs(dip1 - 90.) < EPSIL && fabs(cosdp12) < EPSIL) {
            printf("\nSecond plane dip is indetermined.");
    }
    else {
        dip2 = datan2(cosd(dip1), -sind(dip1) * cosdp12);
    }

    return(dip2);
}

/*********************************************************************/
double computed_rake2(str1, dip1, str2, dip2, fault)

/* 
   Compute rake in the second nodal plane when strike and dip
   for first and second nodal plane are given with a double
   characterizing the fault :
                          +1. inverse fault
                          -1. normal fault.
   Angles are in degrees.
*/

/* Genevieve Patau */

double str1, dip1, str2, dip2, fault;
{
    double rake2, sinrake2;

    double datan2();
  
    if(fabs(dip2 - 90.) < EPSIL)
        sinrake2 = fault * cosd(dip1);
    else
        sinrake2 = -fault * sind(dip1) * cosd(str1 - str2) / cosd(dip2);

    rake2 = datan2(sinrake2, - fault * sind(dip1) * sind(str1 - str2));

    return(rake2);
}

/*********************************************************************/
define_second_plane(NP1, NP2)

/*
    Compute strike, dip, slip for the second nodal plane
    when are given strike, dip and rake for the first one.
*/
/*  Genevieve Patau */

struct nodal_plane NP1;
struct nodal_plane *NP2;
{
    double fault = NP1.rake == 0. ? 1. : NP1.rake / fabs(NP1.rake);

    NP2->str = computed_strike1(NP1);
    NP2->dip = computed_dip1(NP1);
    NP2->rake  = computed_rake1(NP1);
    return;
}

/*********************************************************************/
double null_axis_dip(str1, dip1, str2, dip2)

/* 
   compute null axis dip when strike and dip are given
   for each nodal plane.
   Angles are in degrees.
*/

/* Genevieve Patau */

double str1, dip1, str2, dip2; 
{
    double den;
  
    den = asin(sind(dip1) * sind(dip2) * sind(str1 - str2)) / D2R; 
    if (den < 0.)
        den = -den;
    return(den);
}

/*********************************************************************/
double null_axis_strike(str1, dip1, str2, dip2)

/*
   Compute null axis strike when strike and dip are given
   for each nodal plane.
   Angles are in degrees.
*/

/* Genevieve Patau */

double str1, dip1, str2, dip2;
{
    double phn, cosphn, sinphn;
    double datan2();
    
    cosphn = sind(dip1) * cosd(str1) * cosd(dip2) - sind(dip2) * cosd(str2) * cosd(dip1);
    sinphn = sind(dip1) * sind(str1) * cosd(dip2) - sind(dip2) * sind(str2) * cosd(dip1);
    if(sind(str1 - str2) < 0.) {
        cosphn = -cosphn;
        sinphn = -sinphn;
    } 
    phn = datan2(sinphn, cosphn);
    if(phn < 0.)
        phn += 360.;
    return(phn);
}

/*********************************************************************/
double proj_radius(str1, dip1, str)

/*
   Compute the vector radius for a given strike,
   equal area projection, inferior sphere.
   Strike and dip of the plane are given.
*/

/* Genevieve Patau */

double str1, dip1, str; 
{
    double dip, r;
 
    if(abs(dip1 - 90.) < EPSIL) {
/*
        printf("\nVertical plane : strike is constant.");
        printf("\nFor ps_mechanism r == 1 for str = str1");
        printf("\n            else r == 0. is used.");
*/
        r = (str == str1 || str == str1 + 180) ? 1. : 0.;
    }
    else {
        dip = atan(tand(dip1) * sind(str - str1));
        r = sqrt(2.) * sin(M_PI_4 - dip / 2.);
    }
    return(r);
}


