/*--------------------------------------------------------------------
 *    The GMT-system:	@(#)dat2gmt.c	2.14  05/01/98
 *
 *    Copyright (c) 1991 by P. Wessel and W. H. F. Smith
 *    See README file for copying and redistribution conditions.
 *--------------------------------------------------------------------*/
/*
 * dat2gmt will read an *.dat file (created by gmt2dat), decode it
 * and create a new *.gmt file. To edit a *.gmt file, run gmt2dat
 * to make an ASCII listing, use the editor to change the contents,
 * and then run dat2gmt to create a new binary *.gmtfile.
 *
 * Author:	Paul Wessel
 * Date:	8-FEB-1988
 * Version:	2.0
 *
 */

#include "gmt.h"
#include "gmt_mgg.h"

#define NODATA -32000
#define MEM_SIZE 50000

struct GMT_REC *record;

main (int argc, char **argv)
{
	int rec, n_records, leg_year, time, mag, top, yr, mo, dy, hr, mi, sc, n_alloc = MEM_SIZE, conv;
	double lat, lon, grv;
	char line[BUFSIZ], agency[10], stime[20], sgrv[10], smag[10], stop[10];
	FILE *fpi, *fpo;
	struct GMT *gmt;
	
	if (argc != 3 || ((fpi = fopen(argv[1], "r")) == NULL) || ((fpo = fopen(argv[2], "w")) == NULL)) {
		fprintf(stderr, "usage: dat2gmt asciifile file.gmt\n");
		exit(-1);
	}
	
	record = (struct GMT_REC *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct GMT_REC), "dat2gmt");
	
	/* Read first record of file containing start-year, n_records and info */
	
	fgets (line, BUFSIZ, fpi);
	sscanf (line, "%d %d %s", &leg_year, &n_records, agency);
	gmt = gmt_init (leg_year);
	
	rec = 0;
	while (fgets (line, BUFSIZ, fpi)) {
		sscanf (line, "%s %lf %lf %s %s %s", stime, &lat, &lon, sgrv, smag, stop);
		sscanf (stime, "%d/%d/%d/%d:%d:%d", &yr, &mo, &dy, &hr, &mi, &sc);
		record[rec].time = gmt_time (&time, yr, mo, dy, hr, mi, sc, gmt);
		record[rec].lat = (int) (lat * 1000000.0);
		record[rec].lon = (int) (lon * 1000000.0);
		conv = sscanf(sgrv, "%lf", &grv);
		record[rec].gmt[0] = (conv > 0 && !GMT_is_dnan (grv)) ? (int) rint(grv * 10.0) : NODATA;
		conv = sscanf(smag, "%d", &mag);
		record[rec].gmt[1] = (conv > 0) ? mag : NODATA;
		conv = sscanf(stop, "%d", &top);
		record[rec].gmt[2] = (conv > 0) ? top : NODATA;
		rec++;
		if (rec == n_alloc) {
			n_alloc += GMT_CHUNK;
			record = (struct GMT_REC *) GMT_memory((void *)record, (size_t)n_alloc, sizeof (struct GMT_REC), "dat2gmt");
		}
	}
	fclose (fpi);
	
	free ((void *)gmt);
	n_records = rec;
	if (fwrite((void *)(&leg_year), (size_t)4, (size_t)1, fpo) != 1) {
		fprintf(stderr,"dat2gmt: Error while writing first year\n");
		exit(-1);
	}
	if (fwrite((void *)(&n_records), (size_t)4, (size_t)1, fpo) != 1) {
		fprintf(stderr,"dat2gmt: Error while writing no of records\n");
		exit(-1);
	}
	if (fwrite((void *)agency, (size_t)10, (size_t)1, fpo) != 1) {
		fprintf(stderr,"dat2gmt: Error while writing info-header\n");
		exit(-1);
	}
	for (rec = 0; rec < n_records; rec++) {
		if (fwrite((void *)(&record[rec]), (size_t)18, (size_t)1, fpo) != 1) {
			fprintf(stderr,"dat2gmt: Error writing data record no %d\n",rec);
			exit(-1);
		}
	}
	fclose(fpo);
	
	free ((void *)record);
	
	exit (0);
}
