/*--------------------------------------------------------------------
 *    The GMT-system:	@(#)gmt2dat.c	2.9  03/21/98
 *
 *    Copyright (c) 1991 by P. Wessel and W. H. F. Smith
 *    See README file for copying and redistribution conditions.
 *--------------------------------------------------------------------*/
/*
 * gmt2dat will read a gmt file and create an ascii listing
 * of all the information stored in the *.gmt file. The ASCII
 * file can then be edited manually. Run dat2gmt to put the
 * edited file into *.gmt binary format.
 *
 * Author:	Paul Wessel
 * Date:	8-FEB-1988
 *
 */

#include <stdio.h>
#include <math.h>
#include "gmt_mgg.h"
#include "gmt.h"

#define NODATA -32000

struct GMT_REC *record;

char *memory();

main (int argc, char **argv)
{
	int rec, n_records, leg_year, yr, mo, dy, hr, mi, sc;
	double lat, lon;
	char agency[10];
	FILE *fp;
	struct GMT *gmt;
	
	if (argc != 2 || (fp = fopen(argv[1], "r")) == NULL) {
		fprintf(stderr, "usage: gmt2dat file.gmt > asciifile\n");
		exit (-1);
	}
	
	/* Read first record of file containing start-year, n_records and info */
	
	if (fread((void *)(&leg_year), (size_t)4, (size_t)1, fp) != 1) {
		fprintf(stderr,"gmt2dat: Error while reading first year\n");
		exit(-1);
	}
	if (fread((void *)(&n_records), (size_t)4, (size_t)1, fp) != 1) {
		fprintf(stderr,"gmt2dat: Error while reading no of records\n");
		exit(-1);
	}
	if (fread((void *)agency, (size_t)10, (size_t)1, fp) != 1) {
		fprintf(stderr,"gmt2dat: Error while reading info-header\n");
		exit(-1);
	}
	
	record = (struct GMT_REC *) GMT_memory (VNULL, (size_t)n_records, sizeof (struct GMT_REC), "gmt2dat");
	
	gmt = gmt_init (leg_year);
	
	printf ("%d %d %s\n", leg_year, n_records, agency);
	
	for (rec = 0; rec < n_records; rec++) {
		if (fread ((void *)(&record[rec]), (size_t)18, (size_t)1, fp) != 1) {
			fprintf (stderr,"gmt2dat: Error reading data record no %d\n",rec);
			exit(-1);
		}
	}
	fclose(fp);

	for (rec = 0; rec < n_records; rec++) {
		gmt_date (record[rec].time, &yr, &mo, &dy, &hr, &mi, &sc, gmt);
		lat = (double) record[rec].lat * 0.000001;
		lon = (double) record[rec].lon * 0.000001;
		printf("%d/%.2d/%.2d/%.2d:%.2d:%.2d%10.5lf%10.5lf", yr, mo, dy, hr, mi, sc, lat, lon);
		(record[rec].gmt[0] == NODATA) ? printf ("      NaN") : printf ("%9.2f", (float) record[rec].gmt[0] * 0.1);
		(record[rec].gmt[1] == NODATA) ? printf ("    NaN") : printf ("%7d", record[rec].gmt[1]);
		(record[rec].gmt[2] == NODATA) ? printf ("    NaN\n") : printf ("%7d\n", record[rec].gmt[2]);
	}
	
	free ((void *)gmt);
	free ((void *)record);
	
	exit (0);
}
